@@ -322,11 +322,6 @@ night_light_recheck (GsdNightLight *self)
                         self->disabled_until_tmw = FALSE;
                         g_clear_pointer(&self->disabled_until_tmw_dt, g_date_time_unref);
                         g_object_notify (G_OBJECT (self), "disabled-until-tmw");
-                } else {
-                        g_debug ("night light still day-disabled, resetting");
-                        gsd_night_light_set_temperature (self,
-                                                         GSD_COLOR_TEMPERATURE_DEFAULT);
-                        return;
                 }
         }
 
@@ -343,6 +338,14 @@ night_light_recheck (GsdNightLight *self)
                 return;
         }
 
+        gsd_night_light_set_active (self, TRUE);
+
+        if (self->disabled_until_tmw) {
+                g_debug ("night light still day-disabled");
+                gsd_night_light_set_temperature (self, GSD_COLOR_TEMPERATURE_DEFAULT);
+                return;
+        }
+
         /* smear the temperature for a short duration before the set limits
          *
          *   |----------------------| = from->to
@@ -372,9 +375,9 @@ night_light_recheck (GsdNightLight *self)
         } else {
                 temp_smeared = temperature;
         }
+
         g_debug ("night light mode on, using temperature of %uK (aiming for %uK)",
                  temp_smeared, temperature);
-        gsd_night_light_set_active (self, TRUE);
         gsd_night_light_set_temperature (self, temp_smeared);
 }
 
@@ -701,10 +704,10 @@ gsd_night_light_get_property (GObject    *object,
                 g_value_set_double (value, self->cached_sunrise);
                 break;
         case PROP_SUNSET:
-                g_value_set_double (value, self->cached_sunrise);
+                g_value_set_double (value, self->cached_sunset);
                 break;
         case PROP_TEMPERATURE:
-                g_value_set_double (value, self->cached_sunrise);
+                g_value_set_double (value, self->cached_temperature);
                 break;
         case PROP_DISABLED_UNTIL_TMW:
                 g_value_set_boolean (value, gsd_night_light_get_disabled_until_tmw (self));
